﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Net;
using System.Configuration;
using System.ServiceModel;

namespace DacIEServiceCli
{
    #region DataCenterMapper
    /// <summary>
    /// NOTE: This is NOT an approved way to calculate where a server is located by datacenter
    /// This is brittle to changes in the DNS servers per Microsoft datacenter.
    /// </summary>
    internal class DataCenterMapper
    {
        internal const string Pattern = @"data\.([A-Za-z]{2})([A-Za-z0-9\-]+)\.database\.windows\.net";
        internal static readonly Regex Regex = new Regex(Pattern, RegexOptions.Compiled);
        private static readonly Dictionary<string, DataCenter> DatacenterMap = new Dictionary<string, DataCenter>(StringComparer.InvariantCultureIgnoreCase);

        static DataCenterMapper()
        {
            DataCenterMappingSection section = (DataCenterMappingSection)ConfigurationManager.GetSection("DataCenterMapping");

            if (section == null)
            {
                throw new ArgumentException("Cannot load DataCenterMapping.  Ensure the .config file is in same directory with the executable.");
            }

            DataCenterCollection dccollection = section.DataCenterItems;

            foreach (DataCenter dc in dccollection)
            {
                DatacenterMap.Add(dc.DNSPrefix, dc);
            }
        }
        
        public static DataCenter GetDatacenterFromServerName(string serverName)
        {
            IPHostEntry host = GetHostEntry(serverName);
            string alias = host.HostName;

            if (!host.HostName.EndsWith("INT.MSCDS.COM", StringComparison.InvariantCultureIgnoreCase))
            {
                Match match = Regex.Match(host.HostName);
                if (!match.Success)
                {
                    throw new ArgumentException(string.Format("Unable to detect data center for the server name {0}", serverName));
                }

                alias = match.Groups[1].Value;
            }

            DataCenter dc;
            if (!DataCenterMapper.DatacenterMap.TryGetValue(alias.ToUpperInvariant(), out dc))
            {
                throw new ArgumentException(string.Format("Unable to detect data center for the server name {0}", serverName));
            }

            return dc;
        }

        private static IPHostEntry GetHostEntry(string serverName)
        {
            IPHostEntry host;
            try
            {
                host = Dns.GetHostEntry(serverName);
            }
            catch (Exception ex)
            {
                throw new ArgumentException(string.Format("Unable to detect data center for the server name {0}", serverName), ex);
            }

            return host;
        }
    }
    #endregion

    #region DataCenterMappingSection
    /// <summary>
    /// A custom section for handling the mapping from the app.config
    /// </summary>
    public class DataCenterMappingSection : ConfigurationSection
    {
        [ConfigurationProperty("DataCenters")]
        public DataCenterCollection DataCenterItems
        {
            get
            {
                return ((DataCenterCollection)(base["DataCenters"]));
            }
        }
    }
    #endregion

    #region DataCenterCollection
    [ConfigurationCollection(typeof(DataCenter))]
    public class DataCenterCollection :ConfigurationElementCollection
    {
        protected override ConfigurationElement CreateNewElement()
        {
            return new DataCenter();
        }

        protected override object GetElementKey(ConfigurationElement element)
        {
            return ((DataCenter)(element)).DNSPrefix;
        }

        public DataCenter this[int idx]
        {
            get
            {
                return ((DataCenter)BaseGet(idx));
            }
        }
    }
    #endregion

    #region DataCenterConfigurationElements
    /// <summary>
    /// The actual data center entry in the app.config.  It will look like this:
    /// <add dns="CH1-1" name="North Central US" endpoint="https://ch1prod-dacsvc.azure.com/DACWebService.svc"/>
    /// </summary>
    public class DataCenter : ConfigurationElement
    {
        [ConfigurationProperty("dns", DefaultValue="", IsKey=true, IsRequired=true)]
        public string DNSPrefix
        {
            get
            {
                return ((string)(base["dns"]));
            }
            set
            {
                base["dns"] = value;
            }
        }
        
        [ConfigurationProperty("name", DefaultValue = "", IsKey = false, IsRequired = true)]
        public string Name
        {
            get
            {
                return ((string)(base["name"]));
            }
            set
            {
                base["name"] = value;
            }
        }

        [ConfigurationProperty("endpoint", DefaultValue = "", IsKey = false, IsRequired = true)]
        public string Endpoint
        {
            get
            {
                return ((string)(base["endpoint"]));
            }
            set
            {
                base["endpoint"] = value;
            }
        }

        [ConfigurationProperty("securityMode", DefaultValue = WebHttpSecurityMode.Transport, IsKey = false, IsRequired = false)]
        public WebHttpSecurityMode SecurityMode
        {
            get
            {
                return ((WebHttpSecurityMode)(base["securityMode"]));
            }
            set
            {
                base["securityMode"] = value;
            }
        }
    }
    #endregion
}
